import json
import re

from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories

ACCOUNT_ACCESS = re.compile(r'\d{12}|arn:aws:iam::\d{12}:root')

class IAMRoleAllowAssumeFromAccount(BaseResourceCheck):
    def __init__(self):
        name = "Ensure IAM role allows only specific principals in account to assume it"
        id = "CKV_AWS_61"
        supported_resources = ['AWS::IAM::Role']
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'AssumeRolePolicyDocument' in conf['Properties']:
            assume_role_policy_doc = conf['Properties']['AssumeRolePolicyDocument']
            if isinstance(assume_role_policy_doc, dict) and 'Fn::Sub' in assume_role_policy_doc.keys():
                policy_fn_sub_block = assume_role_policy_doc['Fn::Sub']
                if isinstance(policy_fn_sub_block, list) and len(policy_fn_sub_block) == 2:
                    assume_role_block = json.loads(policy_fn_sub_block[0])
                else:
                    assume_role_block = json.loads(policy_fn_sub_block)
            elif isinstance(assume_role_policy_doc, str):
                assume_role_block = json.loads(assume_role_policy_doc)
            else:
                assume_role_block = assume_role_policy_doc
        else:
            return CheckResult.UNKNOWN

        if 'Statement' in assume_role_block.keys():
            if isinstance(assume_role_block['Statement'], list) and 'Principal' in \
                    assume_role_block['Statement'][0]:
                if 'AWS' in assume_role_block['Statement'][0]['Principal']:
                    if isinstance(assume_role_block['Statement'][0]['Principal']['AWS'],list) \
                            and isinstance(assume_role_block['Statement'][0]['Principal']['AWS'][0], str):
                        if re.match(ACCOUNT_ACCESS, assume_role_block['Statement'][0]['Principal']['AWS'][0]):
                            return CheckResult.FAILED

            return CheckResult.PASSED


check = IAMRoleAllowAssumeFromAccount()



import os

from checkov.common.images.image_referencer import ImageReferencer, Image
from checkov.common.output.report import CheckType
from checkov.github_actions.checks.registry import registry
from checkov.yaml_doc.runner import Runner as YamlRunner

WORKFLOW_DIRECTORY = ".github/workflows/"


class Runner(YamlRunner, ImageReferencer):
    check_type = CheckType.GITHUB_ACTIONS

    def __init__(self):
        super().__init__()

    def require_external_checks(self):
        return False

    def import_registry(self):
        return registry

    def _parse_file(self, f):
        if self.is_workflow_file(f):
            return super()._parse_file(f)

    def is_workflow_file(self, file_path):
        """
        :return: True if the file mentioned is in a github action workflow directory and is a YAML file. Otherwise: False
        """
        abspath = os.path.abspath(file_path)
        return WORKFLOW_DIRECTORY in abspath and abspath.endswith(("yml", "yaml"))

    def get_images(self, file_path):
        """
        Get container images mentioned in a file
        :param file_path: File to be inspected
        GitHub actions workflow file can have a job run within a container.

        in the following sample file we can see a node:14.16 image:

        # jobs:
        #   my_job:
        #     container:
        #       image: node:14.16
        #       env:
        #         NODE_ENV: development
        #       ports:
        #         - 80
        #       volumes:
        #         - my_docker_volume:/volume_mount
        #       options: --cpus 1
        Source: https://docs.github.com/en/actions/using-workflows/workflow-syntax-for-github-actions#example-defining-credentials-for-a-container-registry

        :return: List of container image classes ids mentioned in the file.
        """

        images = set()

        workflow, workflow_line_numbers = self._parse_file(file_path)
        jobs = workflow.get("jobs", {})
        for job_name, job_object in jobs.items():
            if isinstance(job_object, dict):
                container = job_object.get("container", {})
                image = None
                start_line = container.get('__startline__', 0)
                end_line = container.get('__endline__', 0)
                if isinstance(container, dict):
                    image = container.get("image", "")
                elif isinstance(container, str):
                    image = container
                if image:
                    image_id = self.inspect(image)
                    if image_id:
                        image_obj = Image(file_path=file_path, name=image, image_id=image_id, start_line=start_line,
                                          end_line=end_line)
                        images.add(image_obj)

        return images



from typing import Dict, Any, List

from hcl2 import START_LINE, END_LINE

from checkov.terraform.context_parsers.base_parser import BaseContextParser


class DataContextParser(BaseContextParser):
    def __init__(self) -> None:
        definition_type = "data"
        super().__init__(definition_type=definition_type)

    def get_entity_context_path(self, entity_block: Dict[str, Dict[str, Any]]) -> List[str]:
        entity_type, entity_value = next(iter(entity_block.items()))
        entity_name = next(iter(entity_value))
        return [entity_type, entity_name]

    def enrich_definition_block(self, definition_blocks: List[Dict[str, Any]]) -> Dict[str, Any]:
        for entity_block in definition_blocks:
            entity_type, entity_value = next(iter(entity_block.items()))
            entity_name, entity_config = next(iter(entity_value.items()))

            self.context[entity_type][entity_name] = {
                "start_line": entity_config[START_LINE],
                "end_line": entity_config[END_LINE],
                "code_lines": self.file_lines[entity_config[START_LINE] - 1: entity_config[END_LINE]],
            }


        return self.context


parser = DataContextParser()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.dockerfile.base_dockerfile_check import BaseDockerfileCheck


class ExposePort22(BaseDockerfileCheck):
    def __init__(self):
        name = "Ensure port 22 is not exposed"
        id = "CKV_DOCKER_1"
        supported_instructions = ['EXPOSE']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_instructions=supported_instructions)

    def scan_entity_conf(self, conf):
        i = 0
        for expose_term in conf:
            if "22" in expose_term['value'].split(' '):
                return CheckResult.FAILED, conf[i]
            i += 1
        return CheckResult.PASSED , None


check = ExposePort22()



from typing import List, Dict, Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class ECRPolicy(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure ECR policy is not set to public"
        id = "CKV_AWS_32"
        supported_resources = ("aws_ecr_repository_policy",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        """
            Looks for public * policy for ecr repository:
            https://www.terraform.io/docs/providers/aws/r/ecr_repository_policy.html
        :param conf: aws_ecr_repository configuration
        :return: <CheckResult>
        """
        if "policy" in conf.keys():
            policy = conf["policy"][0]
            if type(policy) is str:
                return CheckResult.PASSED
            if policy['Statement'][0] and type(policy['Statement'][0]) is dict:
                statement = policy['Statement'][0]
                if statement['Principal'] and type(statement['Principal']) is str:
                    principal = statement['Principal']
                    if principal == "*" and not self.check_for_constrained_condition(statement):
                        self.evaluated_keys = ["policy/Statement/Principal"]
                        return CheckResult.FAILED
        return CheckResult.PASSED

    def get_evaluated_keys(self) -> List[str]:
        return ['policy']

    def check_for_constrained_condition(self, statement: Dict[str, Any]) -> bool:
        """
        Checks to see if there is a constraint on a a wildcarded principal
        :param statement: statement from aws_repository_configuration
        :return: True if there is a constraint
        """
        if 'Condition' in statement and isinstance(statement['Condition'], dict):
            condition = statement['Condition']
            if 'ForAllValues:StringEquals' in condition and isinstance(condition['ForAllValues:StringEquals'], dict):
                if 'aws:PrincipalOrgID' in condition['ForAllValues:StringEquals']:
                    return True
        return False

check = ECRPolicy()



from typing import List

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck


class DynamoDBTablesEncrypted(BaseResourceCheck):
    def __init__(self):
        name = "Ensure DynamoDB Tables are encrypted using a KMS Customer Managed CMK"
        id = "CKV_AWS_119"
        supported_resources = ["AWS::DynamoDB::Table"]
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        properties = conf.get('Properties')
        if properties is not None:
            sse_config = properties.get('SSESpecification')
            if sse_config is not None:
                sse_enabled = sse_config.get('SSEEnabled')
                sse_key = sse_config.get('KMSMasterKeyId')
                if sse_enabled and sse_key is not None:
                    return CheckResult.PASSED
        return CheckResult.FAILED

    def get_evaluated_keys(self) -> List[str]:
        return ["Properties/SSESpecification/SSEEnabled", "Properties/SSESpecification/KMSMasterKeyId"]


check = DynamoDBTablesEncrypted()



from checkov.common.models.enums import CheckResult

from checkov.github_actions.checks.base_github_action_check import BaseGithubActionsCheck
from checkov.yaml_doc.enums import BlockType


class AllowUnsecureCommandsOnJob(BaseGithubActionsCheck):
    def __init__(self):
        name = "Ensure ACTIONS_ALLOW_UNSECURE_COMMANDS isn't true on environment variables"
        id = "CKV_GHA_1"
        super().__init__(
            name=name,
            id=id,
            block_type=BlockType.ARRAY,
            supported_entities=['jobs', 'jobs.*.steps[]']
        )

    def scan_entity_conf(self, conf):
        if "env" not in conf:
            return CheckResult.PASSED, conf
        env_variables = conf.get("env", {})
        if env_variables.get("ACTIONS_ALLOW_UNSECURE_COMMANDS", False):
            return CheckResult.FAILED, conf
        return CheckResult.PASSED, conf


check = AllowUnsecureCommandsOnJob()



import os
from typing import List, Dict, Any, Tuple

from checkov.cloudformation.graph_builder.graph_components.block_types import BlockType
from checkov.cloudformation.parser import TemplateSections
from checkov.cloudformation.graph_builder.graph_components.blocks import CloudformationBlock


def convert_graph_vertices_to_definitions(
    vertices: List[CloudformationBlock], root_folder: str
) -> Tuple[Dict[str, Dict[str, Any]], Dict[str, Dict[str, Any]]]:
    definitions: Dict[str, Dict[str, Any]] = {}
    breadcrumbs: Dict[str, Dict[str, Any]] = {}
    for vertex in vertices:
        if (vertex.block_type != BlockType.RESOURCE and vertex.block_type != BlockType.PARAMETERS) or \
                (vertex.block_type == BlockType.RESOURCE and not vertex.condition):
            continue
        block_path = vertex.path
        block_type = TemplateSections.RESOURCES.value if vertex.block_type == 'resource' else TemplateSections.PARAMETERS.value
        block_name = vertex.name.split('.')[-1]  # vertex.name is "type.name" so type.name -> [type, name]

        definition = {
            "Type": vertex.attributes["resource_type"] if vertex.block_type == BlockType.RESOURCE else vertex.block_type,
            "Properties": vertex.config or {},
        }

        if vertex.metadata:
            definition["Metadata"] = vertex.metadata

        definitions.setdefault(block_path, {}).setdefault(block_type, {}).setdefault(block_name, definition)

        relative_block_path = f"/{os.path.relpath(block_path, root_folder)}"
        add_breadcrumbs(vertex, breadcrumbs, relative_block_path)
    return definitions, breadcrumbs


def add_breadcrumbs(vertex: CloudformationBlock, breadcrumbs: Dict[str, Dict[str, Any]], relative_block_path: str) -> None:
    vertex_breadcrumbs = vertex.breadcrumbs
    if vertex_breadcrumbs:
        breadcrumbs.setdefault(relative_block_path, {})[vertex.name] = vertex_breadcrumbs



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Callable
from typing_extensions import TypeAlias

from typing_extensions import TypedDict

if TYPE_CHECKING:
    from checkov.common.models.enums import CheckResult
    from checkov.common.checks.base_check import BaseCheck


_ScannerCallableAlias: TypeAlias = Callable[
    [str, "BaseCheck", "_SkippedCheck", "dict[str, Any]", str, str, "dict[str, Any]"], None
]


class _CheckResult(TypedDict, total=False):
    result: "CheckResult"
    suppress_comment: str
    evaluated_keys: list[str]
    results_configuration: dict[str, Any] | None
    check: BaseCheck


class _SkippedCheck(TypedDict, total=False):
    bc_id: str | None
    id: str
    suppress_comment: str



from checkov.kubernetes.checks.resource.base_rbac_check import BaseRbacK8sCheck, RbacOperation


class RbacControlWebhooks(BaseRbacK8sCheck):
    def __init__(self):
        name = "Minimize ClusterRoles that grant control over validating or mutating admission webhook configurations"
        id = "CKV_K8S_155"
        supported_entities = ["ClusterRole"]
        super().__init__(name=name, id=id, supported_entities=supported_entities)

        self.failing_operations = [
            RbacOperation(
                apigroups=["admissionregistration.k8s.io"],
                verbs=["create", "update", "patch"],
                resources=["mutatingwebhookconfigurations", "validatingwebhookconfigurations"]
            ),
        ]


check = RbacControlWebhooks()



from __future__ import annotations

import io
import itertools
import logging
import os
import subprocess  # nosec
import tempfile
from typing import Any, Type, Optional, List
import yaml

from checkov.common.graph.checks_infra.registry import BaseRegistry
from checkov.common.graph.graph_manager import GraphManager
from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
from checkov.common.graph.graph_builder.local_graph import LocalGraph
from checkov.common.output.report import Report, CheckType
from checkov.common.parallelizer.parallel_runner import parallel_runner
from checkov.common.runners.base_runner import BaseRunner, filter_ignored_paths
from checkov.helm.registry import registry
from checkov.kubernetes.graph_builder.local_graph import KubernetesLocalGraph
from checkov.kubernetes.runner import Runner as k8_runner
from checkov.runner_filter import RunnerFilter

K8_POSSIBLE_ENDINGS = [".yaml", ".yml", ".json"]


class K8sHelmRunner(k8_runner):
    check_type = CheckType.HELM

    def __init__(self, graph_class: Type[LocalGraph] = KubernetesLocalGraph,
                 db_connector: NetworkxConnector = NetworkxConnector(),
                 source: str = "Kubernetes",
                 graph_manager: Optional[GraphManager] = None,
                 external_registries: Optional[List[BaseRegistry]] = None) -> None:
        super().__init__(graph_class, db_connector, source, graph_manager, external_registries)
        self.chart_dir_and_meta = []

    def run(self, root_folder: str | None, external_checks_dir: list[str] | None = None, files: list[str] | None = None,
            runner_filter: RunnerFilter = RunnerFilter(), collect_skip_comments: bool = True, helmChart: str = None) -> Report:
        report = Report(self.check_type)
        if not self.chart_dir_and_meta:
            return report
        if external_checks_dir:
            for directory in external_checks_dir:
                registry.load_external_checks(directory)
        for chart_dir, chart_meta in self.chart_dir_and_meta:
            try:
                target_dir = f'{root_folder}{chart_dir}'
                chart_results = super().run(target_dir, external_checks_dir=external_checks_dir,
                                            runner_filter=runner_filter, helmChart=chart_meta['name'])
                fix_report_paths(chart_results, target_dir)
                logging.debug(f"Sucessfully ran k8s scan on {chart_meta['name']}. Scan dir : {target_dir}")
                report.failed_checks.extend(chart_results.failed_checks)
                report.passed_checks.extend(chart_results.passed_checks)
                report.parsing_errors.extend(chart_results.parsing_errors)
                report.skipped_checks.extend(chart_results.skipped_checks)
                report.resources.update(chart_results.resources)

            except Exception:
                logging.warning(f"Failed to run Kubernetes runner on chart {chart_meta['name']}", exc_info=True)
                # with tempfile.TemporaryDirectory() as save_error_dir:
                # TODO this will crash the run when target_dir gets cleaned up, since it no longer exists
                # we either need to copy or find another way to extract whatever we want to get from this (the TODO below)
                # logging.debug(
                #    f"Error running k8s scan on {chart_meta['name']}. Scan dir: {target_dir}. Saved context dir: {save_error_dir}")
                # shutil.move(target_dir, save_error_dir)

                # TODO: Export helm dependancies for the chart we've extracted in chart_dependencies
        return report


class Runner(BaseRunner):
    check_type = CheckType.HELM
    helm_command = 'helm'
    system_deps = True

    def __init__(self):
        super().__init__()
        self.file_names = ['Chart.yaml']
        self.target_folder_path = ''

    def get_k8s_target_folder_path(self) -> str:
        return self.target_folder_path

    @staticmethod
    def find_chart_directories(root_folder: str, files: list[str], excluded_paths: list[str]) -> List[str]:
        chart_directories = []
        if not excluded_paths:
            excluded_paths = []
        if files:
            logging.info('Running with --file argument; checking for Helm Chart.yaml files')
            for file in files:
                if os.path.basename(file) == 'Chart.yaml':
                    chart_directories.append(os.path.dirname(file))

        if root_folder:
            for root, d_names, f_names in os.walk(root_folder):
                filter_ignored_paths(root, d_names, excluded_paths)
                filter_ignored_paths(root, f_names, excluded_paths)
                if 'Chart.yaml' in f_names:
                    chart_directories.append(root)

        return chart_directories

    @staticmethod
    def parse_helm_chart_details(chart_path: str) -> dict[str, Any]:
        with open(f"{chart_path}/Chart.yaml", 'r') as chartyaml:
            try:
                chart_meta = yaml.safe_load(chartyaml)
            except yaml.YAMLError:
                logging.info(f"Failed to load chart metadata from {chart_path}/Chart.yaml.", exc_info=True)
        return chart_meta

    def check_system_deps(self) -> str | None:
        # Ensure local system dependancies are available and of the correct version.
        # Returns framework names to skip if deps fail.
        logging.info(f"Checking necessary system dependancies for {self.check_type} checks.")
        try:
            proc = subprocess.Popen([self.helm_command, 'version'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)  # nosec
            o, e = proc.communicate()
            oString = str(o, 'utf-8')
            if "Version:" in oString:
                helmVersionOutput = oString[oString.find(':') + 2: oString.find(',') - 1]
                if "v3" in helmVersionOutput:
                    logging.info(f"Found working version of {self.check_type} dependancies: {helmVersionOutput}")
                    return None
            else:
                return self.check_type
        except Exception:
            logging.info(f"Error running necessary tools to process {self.check_type} checks.")
            return self.check_type

    def _parse_output(self, target_dir: str, output: bytes) -> None:
        output = str(output, 'utf-8')
        reader = io.StringIO(output)
        cur_source_file = None
        cur_writer = None
        last_line_dashes = False
        line_num = 1
        for s in reader:
            s = s.rstrip()
            if s == '---':
                last_line_dashes = True
                continue

            if last_line_dashes:
                # The next line should contain a "Source" comment saying the name of the file it came from
                # So we will close the old file, open a new file, and write the dashes from last iteration plus this line

                if not s.startswith('# Source: '):
                    raise Exception(f'Line {line_num}: Expected line to start with # Source: {s}')
                source = s[10:]
                if source != cur_source_file:
                    if cur_writer:
                        cur_writer.close()
                    file_path = os.path.join(target_dir, source)
                    parent = os.path.dirname(file_path)
                    os.makedirs(parent, exist_ok=True)
                    cur_source_file = source
                    cur_writer = open(os.path.join(target_dir, source), 'a')
                cur_writer.write('---' + os.linesep)
                cur_writer.write(s + os.linesep)

                last_line_dashes = False
            else:
                if s.startswith('# Source: '):
                    raise Exception(f'Line {line_num}: Unexpected line starting with # Source: {s}')

                if not cur_writer:
                    continue
                else:
                    cur_writer.write(s + os.linesep)

            line_num += 1

        if cur_writer:
            cur_writer.close()

    def convert_helm_to_k8s(self, root_folder: str, files: list[str], runner_filter: RunnerFilter) -> list[tuple[Any, dict[str, Any]]]:
        chart_directories = self.find_chart_directories(root_folder, files, runner_filter.excluded_paths)
        chart_dir_and_meta = list(parallel_runner.run_function(
            lambda cd: (cd, self.parse_helm_chart_details(cd)), chart_directories))
        self.target_folder_path = tempfile.mkdtemp()
        processed_chart_dir_and_meta = []
        for chart_dir, chart_meta in chart_dir_and_meta:
            processed_chart_dir_and_meta.append((chart_dir.replace(root_folder, ""), chart_meta))
            target_dir = chart_dir.replace(root_folder, self.target_folder_path)
            logging.info(
                f"Processing chart found at: {chart_dir}, name: {chart_meta['name']}, version: {chart_meta['version']}")
            # dependency list is nicer to parse than dependency update.
            helm_binary_list_chart_deps = subprocess.Popen([self.helm_command, 'dependency', 'list', chart_dir], stdout=subprocess.PIPE, stderr=subprocess.PIPE)  # nosec
            o, e = helm_binary_list_chart_deps.communicate()
            logging.debug(
                f"Ran helm command to get dependency output. Chart: {chart_meta['name']}. dir: {target_dir}. Output: {str(o, 'utf-8')}. Errors: {str(e, 'utf-8')}")
            if e:
                if "Warning: Dependencies" in str(e, 'utf-8'):
                    logging.info(
                        f"V1 API chart without Chart.yaml dependancies. Skipping chart dependancy list for {chart_meta['name']} at dir: {chart_dir}. Working dir: {target_dir}. Error details: {str(e, 'utf-8')}")
                else:
                    logging.info(
                        f"Error processing helm dependancies for {chart_meta['name']} at source dir: {chart_dir}. Working dir: {target_dir}. Error details: {str(e, 'utf-8')}")

            helm_command_args = [self.helm_command, 'template', '--dependency-update', chart_dir]
            if runner_filter.var_files:
                for var in runner_filter.var_files:
                    helm_command_args.append("--values")
                    helm_command_args.append(var)

            try:
                # --dependency-update needed to pull in deps before templating.
                proc = subprocess.Popen(helm_command_args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)  # nosec
                o, e = proc.communicate()
                logging.debug(
                    f"Ran helm command to template chart output. Chart: {chart_meta['name']}. dir: {target_dir}. Output: {str(o, 'utf-8')}. Errors: {str(e, 'utf-8')}")

            except Exception:
                logging.info(
                    f"Error processing helm chart {chart_meta['name']} at dir: {chart_dir}. Working dir: {target_dir}.",
                    exc_info=True,
                )

            self._parse_output(target_dir, o)
        return processed_chart_dir_and_meta

    def run(self, root_folder: str | None, external_checks_dir: list[str] | None = None, files: list[str] | None = None,
            runner_filter: RunnerFilter = RunnerFilter(), collect_skip_comments: bool = True) -> Report:
        k8s_runner = K8sHelmRunner()
        k8s_runner.chart_dir_and_meta = self.convert_helm_to_k8s(root_folder, files, runner_filter)
        return k8s_runner.run(self.get_k8s_target_folder_path(), external_checks_dir=external_checks_dir, runner_filter=runner_filter)


def fix_report_paths(report: Report, tmp_dir: str) -> None:
    for check in itertools.chain(report.failed_checks, report.passed_checks):
        check.repo_file_path = check.repo_file_path.replace(tmp_dir, '', 1)
        check.file_abs_path = check.file_abs_path.replace(tmp_dir, '', 1)
    report.resources = {r.replace(tmp_dir, '', 1) for r in report.resources}


def get_skipped_checks(entity_conf: dict) -> List:
    skipped = []
    metadata = {}
    if not isinstance(entity_conf, dict):
        return skipped
    if entity_conf["kind"] == "containers" or entity_conf["kind"] == "initContainers":
        metadata = entity_conf["parent_metadata"]
    else:
        if "metadata" in entity_conf.keys():
            metadata = entity_conf["metadata"]
    if "annotations" in metadata.keys() and metadata["annotations"] is not None:
        for key in metadata["annotations"].keys():
            skipped_item = {}
            if "checkov.io/skip" in key or "bridgecrew.io/skip" in key:
                if "CKV_K8S" in metadata["annotations"][key]:
                    if "=" in metadata["annotations"][key]:
                        (skipped_item["id"], skipped_item["suppress_comment"]) = metadata["annotations"][key].split("=")
                    else:
                        skipped_item["id"] = metadata["annotations"][key]
                        skipped_item["suppress_comment"] = "No comment provided"
                    skipped.append(skipped_item)
                else:
                    logging.info(f"Parse of Annotation Failed for {metadata['annotations'][key]}: {entity_conf}")
                    continue
    return skipped



from typing import Dict, List, Any

from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class KinesisFirehoseDeliveryStreamUsesCMK(BaseResourceCheck):

    def __init__(self):
        name = "Ensure that Kinesis Firehose Delivery Streams are encrypted with CMK"
        id = "CKV_AWS_241"
        supported_resources = ["aws_kinesis_firehose_delivery_stream"]
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        if conf.get("kinesis_source_configuration"):
            return CheckResult.UNKNOWN

        if conf.get('server_side_encryption'):
            sse = conf.get('server_side_encryption')[0]
            if sse.get('enabled') != [True]:
                self.evaluated_keys = ['server_side_encryption/[0]/enabled']
                return CheckResult.FAILED
            if sse.get('key_type') != ["CUSTOMER_MANAGED_CMK"]:
                self.evaluated_keys = ['server_side_encryption/[0]/key_type']
                return CheckResult.FAILED
            if not sse.get('key_arn'):
                self.evaluated_keys = ['server_side_encryption/[0]/']
                return CheckResult.FAILED
            if not sse.get('key_arn')[0]:
                self.evaluated_keys = ['server_side_encryption/[0]/key_arn']
                return CheckResult.FAILED
            return CheckResult.PASSED
        return CheckResult.FAILED


check = KinesisFirehoseDeliveryStreamUsesCMK()



import logging
from abc import abstractmethod
from collections.abc import Iterable
from typing import Dict, Any, List, Optional

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.checks.resource.base_spec_check import BaseK8Check
from checkov.kubernetes.checks.resource.registry import registry


class BaseK8sContainerCheck(BaseK8Check):
    TEMPLATE_ENTITIES = (
        "Deployment",
        "DaemonSet",
        "Job",
        "ReplicaSet",
        "ReplicationController",
        "StatefulSet",
    )
    SUPPORTED_ENTITIES = (
        "CronJob",
        "Pod",
        "PodTemplate",
    ) + TEMPLATE_ENTITIES

    def __init__(
        self,
        name: str,
        id: str,
        categories: Optional[List[CheckCategories]] = None,
        supported_entities: Optional["Iterable[str]"] = None,
        supported_container_types: Optional["Iterable[str]"] = None,
        guideline: Optional[str] = None,
    ) -> None:
        categories = categories or [CheckCategories.KUBERNETES]
        supported_entities = supported_entities or BaseK8sContainerCheck.SUPPORTED_ENTITIES

        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=supported_entities,
            guideline=guideline,
        )
        self.supported_container_types = supported_container_types or ("containers", "initContainers")
        self.evaluated_container_keys: List[str] = []

        registry.register(self)

    def get_resource_id(self, conf: Dict[str, Any]) -> str:  # TODO: remove when implementing graph
        return f"{conf['kind']}.{conf['metadata'].get('namespace', 'default')}.{conf['metadata']['name']}"

    def scan_spec_conf(self, conf: Dict[str, Any]) -> CheckResult:
        if self.entity_type == "Pod":
            evaluated_key_prefix = "spec"
            try:
                spec = conf["spec"]
                metadata = conf.get("metadata", {})
            except KeyError:
                logging.info(f"failed to extract {evaluated_key_prefix} for {self.entity_path}")
                return CheckResult.UNKNOWN
        elif self.entity_type in "PodTemplate":
            evaluated_key_prefix = "template/spec"
            try:
                spec = conf["template"]["spec"]
                metadata = conf["template"].get("metadata", {})
            except KeyError:
                logging.info(f"failed to extract {evaluated_key_prefix} for {self.entity_path}")
                return CheckResult.UNKNOWN
        elif self.entity_type in BaseK8sContainerCheck.TEMPLATE_ENTITIES:
            evaluated_key_prefix = "spec/template/spec"
            try:
                spec = conf["spec"]["template"]["spec"]
                metadata = conf["spec"]["template"].get("metadata", {})
            except KeyError:
                logging.info(f"failed to extract {evaluated_key_prefix} for {self.entity_path}")
                return CheckResult.UNKNOWN
        elif self.entity_type == "CronJob":
            evaluated_key_prefix = "spec/jobTemplate/spec/template/spec"
            try:
                spec = conf["spec"]["jobTemplate"]["spec"]["template"]["spec"]
                metadata = conf["spec"]["jobTemplate"]["spec"]["template"].get("metadata", {})
            except KeyError:
                logging.info(f"failed to extract {evaluated_key_prefix} for {self.entity_path}")
                return CheckResult.UNKNOWN
        else:
            logging.info(f"entity type {self.entity_type} not supported")
            return CheckResult.UNKNOWN

        containers: List[Dict[str, Any]] = (
            spec.get("containers", []) if "containers" in self.supported_container_types else []
        ) or []
        init_containers: List[Dict[str, Any]] = (
            spec.get("initContainers", []) if "initContainers" in self.supported_container_types else []
        ) or []

        result = self._check_containers(
            evaluated_key_prefix=evaluated_key_prefix,
            container_type="containers",
            metadata=metadata,
            containers=containers,
        )
        if result == CheckResult.FAILED:
            return CheckResult.FAILED

        result = self._check_containers(
            evaluated_key_prefix=evaluated_key_prefix,
            container_type="initContainers",
            metadata=metadata,
            containers=init_containers,
        )
        if result == CheckResult.FAILED:
            return CheckResult.FAILED

        return CheckResult.PASSED

    def _check_containers(
        self, evaluated_key_prefix: str, container_type: str, metadata: Dict[str, Any], containers: List[Dict[str, Any]]
    ) -> CheckResult:
        """Check containers for possible violations."""
        for idx, container in enumerate(containers):
            result = self.scan_container_conf(metadata, container)

            # fail with the first occurrence
            if result == CheckResult.FAILED:
                if self.evaluated_container_keys:
                    self.evaluated_keys = [
                        f"{evaluated_key_prefix}/{container_type}/[{idx}]/{key}"
                        for key in self.evaluated_container_keys
                    ]
                else:
                    self.evaluated_keys = [f"{evaluated_key_prefix}/initContainers/[{idx}]"]
                return CheckResult.FAILED

        return CheckResult.PASSED

    @abstractmethod
    def scan_container_conf(self, metadata: Dict[str, Any], conf: Dict[str, Any]) -> CheckResult:
        """Return result of container check."""
        pass



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult
from checkov.kubernetes.checks.resource.base_root_container_check import BaseK8sRootContainerCheck


class RootContainersHighUID(BaseK8sRootContainerCheck):
    def __init__(self) -> None:
        name = "Containers should run as a high UID to avoid host conflict"
        # runAsUser should be >= 10000 at pod spec or container level
        # Location: Pod.spec.runAsUser
        # Location: CronJob.spec.jobTemplate.spec.template.spec.securityContext.runAsUser
        # Location: *.spec.template.spec.securityContext.runAsUser
        id = "CKV_K8S_40"
        super().__init__(name=name, id=id)

    def scan_spec_conf(self, conf: dict[str, Any]) -> CheckResult:
        spec = self.extract_spec(conf)

        # Collect results
        if spec:
            results = {"pod": {}, "container": []}
            results["pod"]["runAsUser"] = self.check_runAsUser(spec, 10000)

            if spec.get("containers"):
                for c in spec["containers"]:
                    cresults = {"runAsUser": self.check_runAsUser(c, 10000)}
                    results["container"].append(cresults)

            # Evaluate pass / fail - Container values override Pod values
            # Pod runAsUser >= 10000, no override at container (PASSED)
            # Pod runAsUser >= 10000, override at container < 10000 (FAILED)
            # Pod runAsUser < 10000, no override at container (FAILED)
            # Pod runAsUser < 10000, override at container >= 10000 (PASSED)
            # Pod runAsUser not set, container runAsUser not set or < 10000 (FAILED)
            # Pod runAsUser not set, container runAsUser set >= 10000 (PASSED)
            if results["pod"]["runAsUser"] == "PASSED":
                for cr in results["container"]:
                    if cr["runAsUser"] == "FAILED":
                        return CheckResult.FAILED
                return CheckResult.PASSED
            elif results["pod"]["runAsUser"] == "FAILED":
                containeroverride = False
                for cr in results["container"]:
                    if cr["runAsUser"] == "FAILED" or cr["runAsUser"] == "ABSENT":
                        return CheckResult.FAILED
                    elif cr["runAsUser"] == "PASSED":
                        containeroverride = True
                if containeroverride:
                    return CheckResult.PASSED
                return CheckResult.FAILED
            else:
                # Pod runAsUser ABSENT
                containeroverride = False
                for cr in results["container"]:
                    if cr["runAsUser"] == "FAILED" or cr["runAsUser"] == "ABSENT":
                        return CheckResult.FAILED
                    elif cr["runAsUser"] == "PASSED":
                        containeroverride = True
                if containeroverride:
                    return CheckResult.PASSED
                return CheckResult.FAILED

        return CheckResult.FAILED


check = RootContainersHighUID()
